// puppeteer-extra is a drop-in replacement for puppeteer,
// it augments the installed puppeteer with plugin functionality
const puppeteer = require('puppeteer-extra');
// add stealth plugin and use defaults (all evasion techniques)
const StealthPlugin = require('puppeteer-extra-plugin-stealth');
puppeteer.use(StealthPlugin());

// load helper function to detect stealth plugin
const {warnIfNotUsingStealth, sleep} = require("../helpers/helperFunctions.js");

/**
 * Scrapes all collections from the Rankings page at https://opensea.io/rankings
 * options = {
 *   nbrOfPages: number of pages that should be scraped? (defaults to 1 Page = top 100 collections)
 *   logs: [true,false] show logs in the console
 *   browserInstance: browser instance created with puppeteer.launch() (bring your own puppeteer instance)
 * }
 */
const rankings = async (type = "total", chain = undefined, optionsGiven = {}) => {
	const optionsDefault = {
		debug: false,
		logs: false,
		additionalWait: 0, // waittime in milliseconds, after page loaded, but before stating to scrape
		browserInstance: undefined,
	};
	const options = {...optionsDefault, ...optionsGiven};
	const {logs, additionalWait, browserInstance} = options;
	const customPuppeteerProvided = Boolean(optionsGiven.browserInstance);
	logs && console.log(`=== OpenseaScraper.rankings() ===\n`);

	// init browser
	let browser = browserInstance;
	if (!customPuppeteerProvided) {
		browser = await puppeteer.launch({
			args: ['--start-maximized'],
		});
	}
	customPuppeteerProvided && warnIfNotUsingStealth(browser);

	const page = await browser.newPage();
	const url = getUrl(type, chain);
	logs && console.log("...opening url: " + url);
	await page.goto(url);

	logs && console.log("...🚧 waiting for cloudflare to resolve");
	await page.waitForSelector('.cf-browser-verification', {hidden: true});

	// additional wait?
	if (additionalWait > 0) {
		logs && console.log(`...additional wait active, waiting ${additionalWait / 1000} seconds...`);
		await sleep(additionalWait);
	}

	logs && console.log("...extracting __NEXT_DATA variable");
	const __NEXT_DATA__ = await page.evaluate(() => {
		const nextDataStr = document.getElementById("__NEXT_DATA__").innerText;
		return JSON.parse(nextDataStr);
	});

	// extract relevant info
	const top100 = _parseNextDataVarible(__NEXT_DATA__);
	logs && console.log(`🥳 DONE. Total ${top100.length} Collections fetched: `);
	return top100;
}

function _parseNextDataVarible(__NEXT_DATA__) {
	const initialRecords = __NEXT_DATA__.props.pageProps.initialRecords;
	const getStatsV2 = (node) => {
		if (node.statsV2) {
			return initialRecords[node.statsV2.__ref];
		}
		return {};
	};
	const getWindowCollectionStats = (node) => {
		for (let k in node) {
			if (node.hasOwnProperty(k) && k.startsWith("windowCollectionStats(")) {
				return initialRecords[node[k].__ref];
			}
		}
		return null;
	}
	const extractVolume = (volume) => {
		try {
			return {
				eth: Number(volume.eth),
				unit: Number(volume.unit),
				symbol: volume.symbol.toUpperCase(),
			};
		} catch (err) {
			return null;
		}
	}
	const extractCollection = (node) => {
		const statsV2 = getStatsV2(node);
		const windowCollectionStats = getWindowCollectionStats(node) || {};
		const floorPrice = windowCollectionStats.floorPrice ? initialRecords[windowCollectionStats.floorPrice.__ref] : null;
		const volume = windowCollectionStats.volume ? initialRecords[windowCollectionStats.volume.__ref] : null;
		return {
			createdDate: node.createdDate,
			name: node.name,
			slug: node.slug,
			logo: node.logo,
			imageUrl: node.imageUrl,
			isVerified: node.isVerified,
			totalQuantity: Number(statsV2.totalQuantity),
			floorPrice: extractVolume(floorPrice),
			numOwners: windowCollectionStats.numOwners,
			totalSupply: windowCollectionStats.totalSupply,
			numOfSales: Number(windowCollectionStats.numOfSales),
			volumeChange: windowCollectionStats.volumeChange,
			volume: extractVolume(volume)
		};
	}
	const objects = [];
	for (let k in initialRecords) {
		if (initialRecords.hasOwnProperty(k) && initialRecords[k].edges && initialRecords[k].edges.__refs) {
			const refs = initialRecords[k].edges.__refs;
			for (let i = 0; i < refs.length; ++i) {
				const edge = initialRecords[refs[i]];
				objects.push(initialRecords[edge.node.__ref]);
			}
			break;
		}
	}
	return objects.map(obj => extractCollection(obj));
}

function getUrl(type, chain) {
	chainExtraQueryParameter = chain ? `&chain=${chain}` : ''
	if (type === "1h") {
		return `https://opensea.io/rankings?sortBy=one_our_volume${chainExtraQueryParameter}`;
	} else if (type === "6h") {
		return `https://opensea.io/rankings?sortBy=six_our_volume${chainExtraQueryParameter}`;
	} else if (type === "24h") {
		return `https://opensea.io/rankings?sortBy=one_day_volume${chainExtraQueryParameter}`;
	} else if (type === "7d") {
		return `https://opensea.io/rankings?sortBy=seven_day_volume${chainExtraQueryParameter}`;
	} else if (type === "30d") {
		return `https://opensea.io/rankings?sortBy=thirty_day_volume${chainExtraQueryParameter}`;
	} else if (type === "total") {
		return `https://opensea.io/rankings?sortBy=total_volume${chainExtraQueryParameter}`;
	}

	throw new Error(`Invalid type provided. Expected: 1h,6h,24h,7d,30d,total. Got: ${type}`);
}
module.exports = rankings;

